import { gcj02towgs84, wgs84togcj02 } from './gps';

const AMap = window.AMap;

class NeAMap {
  constructor(dom, opt) {
    const define = {
      zoom: 17,
      center: [ 116.397428, 39.90923 ],
      toolbar: false,
    };
    const option = Object.assign({}, define, opt);
    this.map = new AMap.Map(dom, option);

    AMap.plugin('AMap.Geocoder');

    if (option.toolbar) {
      AMap.plugin([ 'AMap.ToolBar', 'AMap.Scale' ], () => {
        this.map.addControl(new AMap.ToolBar());
        this.map.addControl(new AMap.Scale());
      });
    }
  }

  setMap(opt) {
    const define = {
      zoom: this.map.getZoom(),
      center: this.map.getCenter(),
    };
    const center = opt.center;
    if (center) {
      opt.center = new AMap.LngLat(center[0], center[1]);
    }
    const option = Object.assign(define, opt);
    this.map.setZoomAndCenter(option.zoom, option.center);
  }

  addMarker(pos, opt, cb) {
    const define = {
      image: 'https://webapi.amap.com/theme/v1.3/markers/n/mark_b.png',
      size: new AMap.Size(19, 33),
    };
    const size = opt.size;
    if (size) {
      opt.size = new AMap.Size(size[0], size[1]);
    }
    if (opt.anchor) {
      opt.imageOffset = new AMap.Pixel(opt.anchor[0], opt.anchor[1]);
    } else {
      opt.imageOffset = new AMap.Pixel(size[0] / 2, size[1]);
    }
    const option = Object.assign({}, define, opt);
    const markerObj = {
      map: this.map,
      position: new AMap.LngLat(pos[0], pos[1]),
      icon: new AMap.Icon(option),
    };
    if (option.rotation) {
      markerObj.angle = option.rotation;
    }
    const marker = new AMap.Marker(markerObj);
    if (cb) {
      marker.on('click', cb);
    }
    return marker;
  }

  markerMove(marker, target, moveSpeed) {
    marker.moveTo(new AMap.LngLat(target[0], target[1]), moveSpeed);
  }

  clearMap() {
    this.map.clearMap();
    this.heatmap && this.heatmap.setMap(null);
    this.heatmap = null;
  }

  openInfoWindow(pos, info) {
    const define = {
      content: info,
    };
    const infoWindow = new AMap.InfoWindow(define);
    infoWindow.open(this.map, new AMap.LngLat(pos[0], pos[1]));
  }

  addHeatMap(points, opt) {
    const define = {
      radius: 25,
      opacity: [ 0, 0.8 ],
      gradient: {
        0.4: 'blue',
        0.5: 'rgb(117,211,248)',
        0.6: 'rgb(0, 255, 0)',
        0.7: '#ffea00',
        0.8: 'red',
      },
    };
    const option = Object.assign({}, define, opt);
    this.map.plugin([ 'AMap.Heatmap' ], () => {
      const heatmap = new AMap.Heatmap(this.map, option);
      console.log(heatmap);
      this.heatmap = heatmap;
      heatmap.setDataSet({ data: points, max: 100 });
    });
  }

  showHeatMap() {
    this.heatmap && this.heatmap.show();
  }

  hideHeatMap() {
    this.heatmap && this.heatmap.hide();
  }

  addMoveLine(lineArr, moveSpeed, lineOpt, markerOpt) {
    const pathArr = [];
    for (const item of lineArr) {
      pathArr.push(new AMap.LngLat(item[0], item[1]));
    }
    this.lineArr = pathArr;
    this.moveSpeed = moveSpeed;
    const defineLine = {
      map: this.map,
      path: pathArr,
      strokeColor: '#00A',
      strokeWeight: 3,
    };
    const lineOption = Object.assign({}, defineLine, lineOpt);
    new AMap.Polyline(lineOption);
    const defineMarker = {
      map: this.map,
      offset: new AMap.Pixel(-26, -13),
      icon: 'https://webapi.amap.com/images/car.png',
      autoRotation: true,
    };
    const image = markerOpt.image;
    const size = markerOpt.size;
    if (image) {
      markerOpt.icon = image;
    }
    if (size) {
      markerOpt.offset = new AMap.Pixel(-size[0] / 2, -size[1] / 2);
    }
    const markerOption = Object.assign({}, defineMarker, markerOpt);
    const marker = new AMap.Marker(markerOption);
    this.marker = marker;
    this.map.setFitView();
  }

  startMove() {
    this.marker.moveAlong(this.lineArr, this.moveSpeed);
  }

  pauseMove() {
    this.marker.pauseMove();
  }

  resumeMove() {
    this.marker.resumeMove();
  }

  stopMove() {
    this.marker.stopMove();
  }

  static convert(gps) {
    const latlng = [];
    for (const item of gps) {
      latlng.push(new AMap.LngLat(item[0], item[1]));
    }
    return new Promise((resolve, reject) => {
      AMap.convertFrom(latlng, 'gps', (status, result) => {
        if (status === 'complete' && result.info === 'ok') {
          const location = [];
          for (const item of result.locations) {
            location.push([ item.lng, item.lat ]);
          }
          resolve(location);
        } else {
          reject(result);
        }
      });
    });
  }
  translateFromGPS(data) {
    const transRes = [];
    for (let j = 0; j < data.length; j++) {
      const trans = wgs84togcj02(data[j].latitude, data[j].longitude);
      transRes.push(new AMap.LngLat(trans[1], trans[0]));
    }
    return transRes;
  }
  translateToGPS(data) {
    const transRes = [];
    for (let j = 0; j < data.length; j++) {
      let trans = [];
      if (data[j].getLat) {
        trans = gcj02towgs84(data[j].getLat(), data[j].getLng());
      } else {
        trans = gcj02towgs84(data[j].latitude, data[j].longitude);
      }
      transRes.push({ latitude: trans[0], longitude: trans[1] });
    }
    return transRes;
  }
  static geocode(location) {
    return new Promise((resolve, reject) => {
      new AMap.Geocoder().getAddress(location, (status, result) => {
        if (status === 'complete' && result.info === 'OK') {
          resolve(result.regeocode.formattedAddress);
        } else {
          reject(result);
        }
      });
    });
  }
}

export default NeAMap;
